use rune::Hash;

mod script;
pub mod script_resolver;
mod wrappers;

#[derive(Debug, Default)]
struct RuneScriptType {
    pub fn_on_initialize: Option<Hash>,
    pub fn_on_stack: Option<Hash>,
    pub fn_on_remove: Option<Hash>,
    pub fn_on_before_turn: Option<Hash>,

    pub fn_change_speed: Option<Hash>,
    pub fn_change_priority: Option<Hash>,
    pub fn_change_move: Option<Hash>,
    pub fn_change_number_of_hits: Option<Hash>,
    pub fn_prevent_move: Option<Hash>,
    pub fn_fail_move: Option<Hash>,
    pub fn_stop_before_move: Option<Hash>,
    pub fn_on_before_move: Option<Hash>,
    pub fn_fail_incoming_move: Option<Hash>,
    pub fn_is_invulnerable: Option<Hash>,
    pub fn_on_move_miss: Option<Hash>,
    pub fn_change_move_type: Option<Hash>,
    pub fn_change_effectiveness: Option<Hash>,
    pub fn_block_critical: Option<Hash>,
    pub fn_block_incoming_critical: Option<Hash>,
    pub fn_change_accuracy: Option<Hash>,
    pub fn_change_critical_stage: Option<Hash>,
    pub fn_change_critical_modifier: Option<Hash>,
    pub fn_change_stab_modifier: Option<Hash>,
    pub fn_change_base_power: Option<Hash>,
    pub fn_bypass_defensive_stat_boost: Option<Hash>,
    pub fn_bypass_offensive_stat_boost: Option<Hash>,
    pub fn_change_offensive_stat_value: Option<Hash>,
    pub fn_change_defensive_stat_value: Option<Hash>,
    pub fn_change_damage_stat_modifier: Option<Hash>,
    pub fn_change_damage_modifier: Option<Hash>,
    pub fn_change_damage: Option<Hash>,
    pub fn_change_incoming_damage: Option<Hash>,
    pub fn_on_incoming_hit: Option<Hash>,
    pub fn_on_opponent_faints: Option<Hash>,
    pub fn_prevent_stat_boost_change: Option<Hash>,
    pub fn_change_stat_boost_change: Option<Hash>,
    pub fn_prevent_secondary_effect: Option<Hash>,
    pub fn_change_effect_chance: Option<Hash>,
    pub fn_change_incoming_effect_chance: Option<Hash>,
    pub fn_on_secondary_effect: Option<Hash>,
    pub fn_on_after_hits: Option<Hash>,
    pub fn_prevent_self_switch: Option<Hash>,
    pub fn_prevent_opponent_switch: Option<Hash>,
    pub fn_on_fail: Option<Hash>,
    pub fn_on_opponent_fail: Option<Hash>,
    pub fn_prevent_self_run_away: Option<Hash>,
    pub fn_prevent_opponent_run_away: Option<Hash>,
    pub fn_on_end_turn: Option<Hash>,
    pub fn_on_damage: Option<Hash>,
    pub fn_on_faint: Option<Hash>,
    pub fn_on_switch_in: Option<Hash>,
    pub fn_on_after_held_item_consume: Option<Hash>,
    pub fn_change_experience_gained: Option<Hash>,
    pub fn_share_experience: Option<Hash>,
    pub fn_block_weather: Option<Hash>,
    pub fn_change_capture_rate_bonus: Option<Hash>,
}

impl RuneScriptType {
    pub fn on_found_fn(&mut self, fn_name: &str, hash: Hash) {
        match fn_name {
            "on_initialize" => self.fn_on_initialize = Some(hash),
            "on_stack" => self.fn_on_stack = Some(hash),
            "on_remove" => self.fn_on_remove = Some(hash),
            "on_before_turn" => self.fn_on_before_turn = Some(hash),
            "change_speed" => self.fn_change_speed = Some(hash),
            "change_priority" => self.fn_change_priority = Some(hash),
            "change_move" => self.fn_change_move = Some(hash),
            "change_number_of_hits" => self.fn_change_number_of_hits = Some(hash),
            "prevent_move" => self.fn_prevent_move = Some(hash),
            "fail_move" => self.fn_fail_move = Some(hash),
            "stop_before_move" => self.fn_stop_before_move = Some(hash),
            "on_before_move" => self.fn_on_before_move = Some(hash),
            "fail_incoming_move" => self.fn_fail_incoming_move = Some(hash),
            "is_invulnerable" => self.fn_is_invulnerable = Some(hash),
            "on_move_miss" => self.fn_on_move_miss = Some(hash),
            "change_move_type" => self.fn_change_move_type = Some(hash),
            "change_effectiveness" => self.fn_change_effectiveness = Some(hash),
            "block_critical" => self.fn_block_critical = Some(hash),
            "block_incoming_critical" => self.fn_block_incoming_critical = Some(hash),
            "change_accuracy" => self.fn_change_accuracy = Some(hash),
            "change_critical_stage" => self.fn_change_critical_stage = Some(hash),
            "change_critical_modifier" => self.fn_change_critical_modifier = Some(hash),
            "change_stab_modifier" => self.fn_change_stab_modifier = Some(hash),
            "change_base_power" => self.fn_change_base_power = Some(hash),
            "bypass_defensive_stat_boost" => self.fn_bypass_defensive_stat_boost = Some(hash),
            "bypass_offensive_stat_boost" => self.fn_bypass_offensive_stat_boost = Some(hash),
            "change_offensive_stat_value" => self.fn_change_offensive_stat_value = Some(hash),
            "change_defensive_stat_value" => self.fn_change_defensive_stat_value = Some(hash),
            "change_damage_stat_modifier" => self.fn_change_damage_stat_modifier = Some(hash),
            "change_damage_modifier" => self.fn_change_damage_modifier = Some(hash),
            "change_damage" => self.fn_change_damage = Some(hash),
            "change_incoming_damage" => self.fn_change_incoming_damage = Some(hash),
            "on_incoming_hit" => self.fn_on_incoming_hit = Some(hash),
            "on_opponent_faints" => self.fn_on_opponent_faints = Some(hash),
            "prevent_stat_boost_change" => self.fn_prevent_stat_boost_change = Some(hash),
            "change_stat_boost_change" => self.fn_change_stat_boost_change = Some(hash),
            "prevent_secondary_effect" => self.fn_prevent_secondary_effect = Some(hash),
            "change_effect_chance" => self.fn_change_effect_chance = Some(hash),
            "change_incoming_effect_chance" => self.fn_change_incoming_effect_chance = Some(hash),
            "on_secondary_effect" => self.fn_on_secondary_effect = Some(hash),
            "on_after_hits" => self.fn_on_after_hits = Some(hash),
            "prevent_self_switch" => self.fn_prevent_self_switch = Some(hash),
            "prevent_opponent_switch" => self.fn_prevent_opponent_switch = Some(hash),
            "on_fail" => self.fn_on_fail = Some(hash),
            "on_opponent_fail" => self.fn_on_opponent_fail = Some(hash),
            "prevent_self_run_away" => self.fn_prevent_self_run_away = Some(hash),
            "prevent_opponent_run_away" => self.fn_prevent_opponent_run_away = Some(hash),
            "on_end_turn" => self.fn_on_end_turn = Some(hash),
            "on_damage" => self.fn_on_damage = Some(hash),
            "on_faint" => self.fn_on_faint = Some(hash),
            "on_switch_in" => self.fn_on_switch_in = Some(hash),
            "on_after_held_item_consume" => self.fn_on_after_held_item_consume = Some(hash),
            "change_experience_gained" => self.fn_change_experience_gained = Some(hash),
            "share_experience" => self.fn_share_experience = Some(hash),
            "block_weather" => self.fn_block_weather = Some(hash),
            "change_capture_rate_bonus" => self.fn_change_capture_rate_bonus = Some(hash),
            _ => {}
        }
    }
}
