use crate::dynamic_data::BattleRandom;
use crate::script_implementations::rune::wrappers::dynamic_data::executing_move::RuneExecutingMove;
use crate::script_implementations::rune::wrappers::dynamic_data::pokemon::RunePokemon;
use crate::script_implementations::rune::wrappers::impl_rune_wrapper;
use rune::Any;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneBattleRandom>()?;
    module.function_meta(RuneBattleRandom::get)?;
    module.function_meta(RuneBattleRandom::get_max)?;
    module.function_meta(RuneBattleRandom::get_between)?;
    module.function_meta(RuneBattleRandom::effect_chance)?;

    Ok(())
}

#[derive(Debug, Clone, Any)]
pub struct RuneBattleRandom(pub(crate) Arc<BattleRandom>);

impl_rune_wrapper!(&Arc<BattleRandom>, RuneBattleRandom);

impl RuneBattleRandom {
    #[rune::function]
    fn get(&self) -> i32 { self.0.get().unwrap() }

    #[rune::function]
    fn get_max(&self, max: i32) -> i32 { self.0.get_max(max).unwrap() }

    #[rune::function]
    fn get_between(&self, min: i32, max: i32) -> i32 { self.0.get_between(min, max).unwrap() }

    #[rune::function]
    fn effect_chance(
        &self,
        chance: f32,
        executing_move: &RuneExecutingMove,
        target: RunePokemon,
        hit_number: u8,
    ) -> anyhow::Result<bool> {
        self.0.effect_chance(chance, &executing_move.0, &target.0, hit_number)
    }
}
