use crate::dynamic_data::{ExecutingMove, HitData};
use crate::script_implementations::rune::wrappers::dynamic_data::pokemon::RunePokemon;
use crate::script_implementations::rune::wrappers::dynamic_data::resolve_script_data;
use crate::script_implementations::rune::wrappers::{impl_rune_wrapper, RuneWrapper};
use rune::runtime::Value;
use rune::Any;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneExecutingMove>()?;
    module.function_meta(RuneExecutingMove::target_count)?;
    module.function_meta(RuneExecutingMove::number_of_hits)?;
    module.function_meta(RuneExecutingMove::user)?;
    module.function_meta(RuneExecutingMove::chosen_move)?;
    module.function_meta(RuneExecutingMove::use_move)?;
    module.function_meta(RuneExecutingMove::script)?;

    Ok(())
}

#[derive(Debug, Clone, Any)]
pub struct RuneExecutingMove(pub Arc<ExecutingMove>);

impl_rune_wrapper!(&Arc<ExecutingMove>, RuneExecutingMove);

impl RuneExecutingMove {
    #[rune::function]
    fn target_count(&self) -> usize { self.0.target_count() }
    #[rune::function]
    fn number_of_hits(&self) -> u8 { self.0.number_of_hits() }
    #[rune::function]
    fn user(&self) -> Value { self.0.user().wrap() }

    #[rune::function]
    fn chosen_move(&self) -> Value { self.0.chosen_move().wrap() }

    #[rune::function]
    fn use_move(&self) -> Value { self.0.use_move().wrap() }

    #[rune::function]
    fn script(&self) -> Option<Value> { resolve_script_data(self.0.script()) }

    #[rune::function]
    fn get_hit_data(&self, for_target: RunePokemon, hit: u8) -> anyhow::Result<Value> {
        self.0.get_hit_data(&for_target.0, hit).map(|hit_data| hit_data.wrap())
    }
}

#[derive(Debug, Clone, Any)]
pub struct RuneHitData(pub Arc<HitData>);

impl_rune_wrapper!(&Arc<HitData>, RuneHitData);

impl RuneHitData {
    #[rune::function]
    fn is_critical(&self) -> bool { self.0.is_critical() }

    #[rune::function]
    fn base_power(&self) -> u8 { self.0.base_power() }

    #[rune::function]
    fn effectiveness(&self) -> f32 { self.0.effectiveness() }

    #[rune::function]
    fn damage(&self) -> u32 { self.0.damage() }

    #[rune::function]
    fn move_type(&self) -> u8 { u8::from(self.0.move_type()) }

    #[rune::function]
    fn fail(&self) { self.0.fail() }
}
