use crate::dynamic_data::{LearnedMove, MoveLearnMethod};
use crate::script_implementations::rune::wrappers::{impl_rune_wrapper, RuneWrapper};
use rune::runtime::Value;
use rune::Any;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneLearnedMove>()?;
    module.function_meta(RuneLearnedMove::move_data)?;
    module.function_meta(RuneLearnedMove::max_pp)?;
    module.function_meta(RuneLearnedMove::max_pp_modification)?;
    module.function_meta(RuneLearnedMove::remaining_pp)?;
    module.function_meta(RuneLearnedMove::learn_method)?;
    module.function_meta(RuneLearnedMove::try_use)?;
    module.function_meta(RuneLearnedMove::restore_all_uses)?;
    module.function_meta(RuneLearnedMove::restore_uses)?;
    Ok(())
}

#[derive(Debug, Clone, Any)]
struct RuneLearnedMove(Arc<LearnedMove>);

impl_rune_wrapper!(&Arc<LearnedMove>, RuneLearnedMove);

impl RuneLearnedMove {
    #[rune::function]
    fn move_data(&self) -> Value { self.0.move_data().wrap() }

    #[rune::function]
    fn max_pp(&self) -> u8 { self.0.max_pp() }

    #[rune::function]
    fn max_pp_modification(&self) -> u8 { self.0.max_pp_modification() }

    #[rune::function]
    fn remaining_pp(&self) -> u8 { self.0.remaining_pp() }

    #[rune::function]
    fn learn_method(&self) -> MoveLearnMethod { self.0.learn_method() }

    #[rune::function]
    fn try_use(&self, amount: u8) -> bool { self.0.try_use(amount) }

    #[rune::function]
    fn restore_all_uses(&self) { self.0.restore_all_uses() }

    #[rune::function]
    fn restore_uses(&self, amount: u8) { self.0.restore_uses(amount) }
}
