use crate::dynamic_data::{BattleStatCalculator, DynamicLibrary, MiscLibrary};
use crate::script_implementations::rune::wrappers::dynamic_data::pokemon::RunePokemon;
use crate::script_implementations::rune::wrappers::dynamic_data::turn_choice::RuneTurnChoice;
use crate::script_implementations::rune::wrappers::impl_rune_wrapper;
use crate::script_implementations::rune::wrappers::static_data::libraries::static_data::RuneStaticData;
use crate::static_data::{Statistic, TimeOfDay};
use rune::Any;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneBattleStatCalculator>()?;
    module.function_meta(RuneBattleStatCalculator::calculate_flat_stat)?;
    module.function_meta(RuneBattleStatCalculator::calculate_boosted_stat)?;

    module.ty::<RuneMiscLibrary>()?;
    module.function_meta(RuneMiscLibrary::can_flee)?;
    module.function_meta(RuneMiscLibrary::time_of_day)?;

    module.ty::<RuneDynamicLibrary>()?;
    module.function_meta(RuneDynamicLibrary::battle_stat_calculator)?;
    module.function_meta(RuneDynamicLibrary::misc_library)?;

    Ok(())
}

#[derive(Debug, Clone, Any)]
pub struct RuneBattleStatCalculator(Arc<dyn BattleStatCalculator>);

impl_rune_wrapper!(&Arc<dyn BattleStatCalculator>, RuneBattleStatCalculator);

impl RuneBattleStatCalculator {
    #[rune::function]
    fn calculate_flat_stat(&self, pokemon: RunePokemon, stat: Statistic) -> anyhow::Result<u32> {
        self.0.calculate_flat_stat(&pokemon.0, stat)
    }

    #[rune::function]
    fn calculate_boosted_stat(&self, pokemon: RunePokemon, stat: Statistic) -> anyhow::Result<u32> {
        self.0.calculate_boosted_stat(&pokemon.0, stat)
    }
}

#[derive(Debug, Clone, Any)]
pub struct RuneMiscLibrary(Arc<dyn MiscLibrary>);
impl_rune_wrapper!(&Arc<dyn MiscLibrary>, RuneMiscLibrary);

impl RuneMiscLibrary {
    #[rune::function]
    fn can_flee(&self, choice: RuneTurnChoice) -> bool { self.0.can_flee(choice.get_turn_choice()) }

    #[rune::function]
    fn time_of_day(&self) -> TimeOfDay { self.0.time_of_day() }
}

#[derive(Debug, Clone, Any)]
pub struct RuneDynamicLibrary(Arc<dyn DynamicLibrary>);
impl_rune_wrapper!(&Arc<dyn DynamicLibrary>, RuneDynamicLibrary);

impl RuneDynamicLibrary {
    #[rune::function]
    fn battle_stat_calculator(&self) -> RuneBattleStatCalculator {
        RuneBattleStatCalculator(self.0.stat_calculator().clone())
    }

    #[rune::function]
    fn misc_library(&self) -> RuneMiscLibrary { RuneMiscLibrary(self.0.misc_library().clone()) }

    #[rune::function]
    fn static_data(&self) -> RuneStaticData { RuneStaticData(self.0.static_data().clone()) }
}
