use crate::dynamic_data::ScriptContainer;
use crate::script_implementations::rune::script::RuneScript;

mod battle;
mod battle_party;
mod battle_random;
mod battle_side;
mod executing_move;
mod learned_move;
mod libraries;
mod pokemon;
mod turn_choice;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    executing_move::register(module)?;
    pokemon::register(module)?;
    turn_choice::register(module)?;
    learned_move::register(module)?;
    battle::register(module)?;
    battle_random::register(module)?;
    battle_party::register(module)?;
    battle_side::register(module)?;

    libraries::register(module)?;
    Ok(())
}

fn resolve_script_data(container: &ScriptContainer) -> Option<rune::Value> {
    container
        .get()
        .map(|v| {
            v.read()
                .as_ref()
                .map(|v| v.clone().as_any().downcast_ref::<RuneScript>().map(|s| s.get_state()))
        })
        .flatten()
        .flatten()
}
