use crate::script_implementations::rune::wrappers::{RuneStringKey, RuneWrapper};
use crate::static_data::Parameter;
use rune::runtime::Value;
use rune::Any;
use std::ops::Deref;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneParameter>()?;
    Ok(())
}

#[derive(Debug, Any)]
#[rune(name = Parameter)]
pub(super) enum RuneParameter {
    Int(#[rune(get)] i32),
    Float(#[rune(get)] f32),
    String(#[rune(get)] RuneStringKey),
    Bool(#[rune(get)] bool),
}

impl RuneWrapper for &Arc<Parameter> {
    fn wrap(self) -> Value {
        let p = match self.deref() {
            Parameter::Bool(b) => RuneParameter::Bool(*b),
            Parameter::Int(i) => RuneParameter::Int(*i as i32),
            Parameter::Float(f) => RuneParameter::Float(*f),
            Parameter::String(s) => RuneParameter::String(RuneStringKey::new(s.clone())),
        };
        rune::to_value(p).unwrap()
    }
}
