use crate::defines::LevelInt;
use crate::script_implementations::rune::wrappers::{impl_rune_wrapper, RuneStringKey, RuneWrapper};
use crate::static_data::LearnableMoves;
use rune::runtime::Value;
use rune::Any;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneLearnableMoves>()?;
    module.function_meta(RuneLearnableMoves::get_learned_by_level)?;
    module.function_meta(RuneLearnableMoves::get_distinct_level_moves)?;
    module.function_meta(RuneLearnableMoves::learns_move)?;
    Ok(())
}

#[derive(Debug, Any)]
pub struct RuneLearnableMoves(Arc<dyn LearnableMoves>);

impl_rune_wrapper!(&Arc<dyn LearnableMoves>, RuneLearnableMoves);

impl RuneLearnableMoves {
    #[rune::function]
    fn get_learned_by_level(&self, level: LevelInt) -> Option<Vec<Value>> {
        self.0
            .get_learned_by_level(level)
            .map(|v| v.into_iter().map(|s| s.wrap()).collect())
    }

    #[rune::function]
    fn get_distinct_level_moves(&self) -> Vec<Value> {
        self.0
            .get_distinct_level_moves()
            .into_iter()
            .map(|s| s.wrap())
            .collect()
    }

    #[rune::function]
    fn learns_move(&self, moveName: &RuneStringKey) -> bool { self.0.get_distinct_level_moves().contains(&moveName.0) }
}
