use crate::defines::LevelInt;
use crate::script_implementations::rune::wrappers::{impl_rune_wrapper, RuneStringKey};
use crate::static_data::GrowthRateLibrary;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneGrowthRateLibrary>()?;
    module.function_meta(RuneGrowthRateLibrary::calculate_level)?;
    module.function_meta(RuneGrowthRateLibrary::calculate_experience)?;

    Ok(())
}

#[derive(Debug, rune::Any)]
struct RuneGrowthRateLibrary(Arc<dyn GrowthRateLibrary>);

impl_rune_wrapper!(&Arc<dyn GrowthRateLibrary>, RuneGrowthRateLibrary);

impl RuneGrowthRateLibrary {
    #[rune::function]
    fn calculate_level(&self, growth_rate: &RuneStringKey, experience: u32) -> anyhow::Result<LevelInt> {
        self.0.calculate_level(&growth_rate.0, experience)
    }

    #[rune::function]
    fn calculate_experience(&self, growth_rate: &RuneStringKey, level: LevelInt) -> anyhow::Result<u32> {
        self.0.calculate_experience(&growth_rate.0, level)
    }
}
