mod growth_rate_library;
mod library_settings;
mod nature_library;
pub mod static_data;
mod type_library;

use crate::script_implementations::rune::wrappers::{impl_rune_wrapper, RuneStringKey, RuneWrapper};
use crate::static_data::{AbilityLibrary, ItemLibrary, MoveLibrary, SpeciesLibrary};
use rune::runtime::Value;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneAbilityLibrary>()?;
    module.function_meta(RuneAbilityLibrary::get)?;
    module.function_meta(RuneAbilityLibrary::len)?;
    module.function_meta(RuneAbilityLibrary::get_key_by_index)?;

    module.ty::<RuneItemLibrary>()?;
    module.function_meta(RuneItemLibrary::get)?;
    module.function_meta(RuneItemLibrary::len)?;
    module.function_meta(RuneItemLibrary::get_key_by_index)?;

    module.ty::<RuneMoveLibrary>()?;
    module.function_meta(RuneMoveLibrary::get)?;
    module.function_meta(RuneMoveLibrary::len)?;
    module.function_meta(RuneMoveLibrary::get_key_by_index)?;

    module.ty::<RuneSpeciesLibrary>()?;
    module.function_meta(RuneSpeciesLibrary::get)?;
    module.function_meta(RuneSpeciesLibrary::len)?;
    module.function_meta(RuneSpeciesLibrary::get_key_by_index)?;

    growth_rate_library::register(module)?;
    library_settings::register(module)?;
    nature_library::register(module)?;
    type_library::register(module)?;

    static_data::register(module)?;

    Ok(())
}

macro_rules! impl_rune_data_library_wrapper {
    ($t:ident, $wrapped_type:ty) => {
        #[derive(Debug, rune::Any)]
        struct $t($wrapped_type);

        impl_rune_wrapper!(&$wrapped_type, $t);

        impl $t {
            #[rune::function]
            fn get(&self, key: &RuneStringKey) -> Option<Value> { self.0.get(&key.0).map(|v| v.wrap()) }

            #[rune::function]
            fn len(&self) -> usize { self.0.len() }

            #[rune::function]
            fn get_key_by_index(&self, index: usize) -> Option<Value> {
                self.0.get_key_by_index(index).map(|v| v.wrap())
            }
        }
    };
}

impl_rune_data_library_wrapper!(RuneAbilityLibrary, Arc<dyn AbilityLibrary>);
impl_rune_data_library_wrapper!(RuneItemLibrary, Arc<dyn ItemLibrary>);
impl_rune_data_library_wrapper!(RuneMoveLibrary, Arc<dyn MoveLibrary>);
impl_rune_data_library_wrapper!(RuneSpeciesLibrary, Arc<dyn SpeciesLibrary>);
