use crate::script_implementations::rune::wrappers::{impl_rune_wrapper, RuneWrapper};
use crate::static_data::StaticData;
use rune::runtime::Value;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneStaticData>()?;
    module.function_meta(RuneStaticData::settings)?;
    module.function_meta(RuneStaticData::species)?;
    module.function_meta(RuneStaticData::moves)?;
    module.function_meta(RuneStaticData::items)?;
    module.function_meta(RuneStaticData::growth_rates)?;
    module.function_meta(RuneStaticData::types)?;
    module.function_meta(RuneStaticData::natures)?;
    module.function_meta(RuneStaticData::abilities)?;
    Ok(())
}

#[derive(Debug, rune::Any)]
pub struct RuneStaticData(pub Arc<dyn StaticData>);

impl_rune_wrapper!(&Arc<dyn StaticData>, RuneStaticData);

impl RuneStaticData {
    #[rune::function]
    fn settings(&self) -> Value { self.0.settings().wrap() }

    #[rune::function]
    fn species(&self) -> Value { self.0.species().wrap() }

    #[rune::function]
    fn moves(&self) -> Value { self.0.moves().wrap() }

    #[rune::function]
    fn items(&self) -> Value { self.0.items().wrap() }

    #[rune::function]
    fn growth_rates(&self) -> Value { self.0.growth_rates().wrap() }

    #[rune::function]
    fn types(&self) -> Value { self.0.types().wrap() }

    #[rune::function]
    fn natures(&self) -> Value { self.0.natures().wrap() }

    #[rune::function]
    fn abilities(&self) -> Value { self.0.abilities().wrap() }
}
