use crate::script_implementations::rune::wrappers::{impl_rune_wrapper, RuneStringKey, RuneWrapper};
use crate::static_data::TypeLibrary;
use rune::runtime::Value;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneTypeLibrary>()?;
    module.function_meta(RuneTypeLibrary::get_type_id)?;
    module.function_meta(RuneTypeLibrary::get_type_name)?;
    module.function_meta(RuneTypeLibrary::get_single_effectiveness)?;
    module.function_meta(RuneTypeLibrary::get_effectiveness)?;
    Ok(())
}

#[derive(Debug, rune::Any)]
struct RuneTypeLibrary(Arc<dyn TypeLibrary>);

impl_rune_wrapper!(&Arc<dyn TypeLibrary>, RuneTypeLibrary);

impl RuneTypeLibrary {
    #[rune::function]
    fn get_type_id(&self, key: &RuneStringKey) -> Option<u8> { self.0.get_type_id(&key.0).map(|v| v.into()) }

    #[rune::function]
    fn get_type_name(&self, t: u8) -> Option<Value> { self.0.get_type_name(t.into()).map(|v| v.wrap()) }

    #[rune::function]
    fn get_single_effectiveness(&self, attacking: u8, defending: u8) -> anyhow::Result<f32> {
        self.0.get_single_effectiveness(attacking.into(), defending.into())
    }

    #[rune::function]
    fn get_effectiveness(&self, attacking: u8, defending: &[u8]) -> anyhow::Result<f32> {
        self.0.get_effectiveness(
            attacking.into(),
            &defending.iter().map(|v| (*v).into()).collect::<Vec<_>>(),
        )
    }
}
