use crate::script_implementations::rune::wrappers::{impl_rune_wrapper, RuneStringKey, RuneWrapper};
use crate::static_data::Species;
use rune::runtime::Value;
use rune::Any;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneSpecies>()?;
    module.function_meta(RuneSpecies::id)?;
    module.function_meta(RuneSpecies::name)?;
    module.function_meta(RuneSpecies::gender_rate)?;
    module.function_meta(RuneSpecies::growth_rate)?;
    module.function_meta(RuneSpecies::capture_rate)?;
    module.function_meta(RuneSpecies::base_happiness)?;
    module.function_meta(RuneSpecies::get_form)?;
    module.function_meta(RuneSpecies::get_default_form)?;
    module.function_meta(RuneSpecies::has_flag)?;

    Ok(())
}

#[derive(Debug, Any)]
pub struct RuneSpecies(Arc<dyn Species>);

impl_rune_wrapper!(&Arc<dyn Species>, RuneSpecies);

impl RuneSpecies {
    #[rune::function]
    fn id(&self) -> u16 { self.0.id() }

    #[rune::function]
    fn name(&self) -> Value { self.0.name().wrap() }

    #[rune::function]
    fn gender_rate(&self) -> f32 { self.0.gender_rate() }

    #[rune::function]
    fn growth_rate(&self) -> Value { self.0.growth_rate().wrap() }

    #[rune::function]
    fn capture_rate(&self) -> u8 { self.0.capture_rate() }

    #[rune::function]
    fn base_happiness(&self) -> u8 { self.0.base_happiness() }

    #[rune::function]
    fn get_form(&self, name: &RuneStringKey) -> Option<Value> { self.0.get_form(&name.0).map(|form| form.wrap()) }

    #[rune::function]
    fn get_default_form(&self) -> anyhow::Result<Value> { self.0.get_default_form().map(|v| v.wrap()) }

    #[rune::function]
    fn has_flag(&self, key: &RuneStringKey) -> bool { self.0.has_flag(&key.0) }
}
