use crate::script_implementations::rune::wrappers::impl_rune_wrapper;
use crate::static_data::{StaticStatisticSet, Statistic, StatisticSet};
use rune::Any;
use std::sync::Arc;

pub fn register(module: &mut rune::Module) -> anyhow::Result<()> {
    module.ty::<RuneU32StatisticSet>()?;
    module.function_meta(RuneU32StatisticSet::get)?;
    module.function_meta(RuneU32StatisticSet::set)?;
    module.function_meta(RuneU32StatisticSet::hp)?;
    module.function_meta(RuneU32StatisticSet::attack)?;
    module.function_meta(RuneU32StatisticSet::defense)?;
    module.function_meta(RuneU32StatisticSet::special_attack)?;
    module.function_meta(RuneU32StatisticSet::special_defense)?;
    module.function_meta(RuneU32StatisticSet::speed)?;

    module.ty::<RuneStaticStatisticSet>()?;
    module.function_meta(RuneStaticStatisticSet::get)?;
    module.function_meta(RuneStaticStatisticSet::hp)?;
    module.function_meta(RuneStaticStatisticSet::attack)?;
    module.function_meta(RuneStaticStatisticSet::defense)?;
    module.function_meta(RuneStaticStatisticSet::special_attack)?;
    module.function_meta(RuneStaticStatisticSet::special_defense)?;
    module.function_meta(RuneStaticStatisticSet::speed)?;

    Ok(())
}

#[derive(Debug, Any)]
pub struct RuneU32StatisticSet(Arc<StatisticSet<u32>>);

impl_rune_wrapper!(&Arc<StatisticSet<u32>>, RuneU32StatisticSet);
impl RuneU32StatisticSet {
    #[rune::function]
    fn get(&self, stat: Statistic) -> u32 { self.0.get_stat(stat) }

    #[rune::function]
    fn set(&mut self, stat: Statistic, value: u32) { self.0.set_stat(stat, value) }

    #[rune::function]
    fn hp(&self) -> u32 { self.0.hp() }

    #[rune::function]
    fn attack(&self) -> u32 { self.0.attack() }

    #[rune::function]
    fn defense(&self) -> u32 { self.0.defense() }

    #[rune::function]
    fn special_attack(&self) -> u32 { self.0.special_attack() }

    #[rune::function]
    fn special_defense(&self) -> u32 { self.0.special_defense() }

    #[rune::function]
    fn speed(&self) -> u32 { self.0.speed() }
}

#[derive(Debug, Any)]
pub struct RuneStaticStatisticSet(Arc<StaticStatisticSet<u16>>);

impl_rune_wrapper!(&Arc<StaticStatisticSet<u16>>, RuneStaticStatisticSet);

impl RuneStaticStatisticSet {
    #[rune::function]
    fn get(&self, stat: Statistic) -> u16 { self.0.get_stat(stat) }

    #[rune::function]
    fn hp(&self) -> u16 { self.0.hp() }

    #[rune::function]
    fn attack(&self) -> u16 { self.0.attack() }

    #[rune::function]
    fn defense(&self) -> u16 { self.0.defense() }

    #[rune::function]
    fn special_attack(&self) -> u16 { self.0.special_attack() }

    #[rune::function]
    fn special_defense(&self) -> u16 { self.0.special_defense() }

    #[rune::function]
    pub fn speed(&self) -> u16 { self.0.speed() }
}
