use std::intrinsics::transmute;

use crate::dynamic_data::{LearnedMove, MoveLearnMethod};
use crate::script_implementations::wasm::export_registry::wasm_result::{
    get_value_arc, get_value_arc_void, wasm_ok, WasmVoidResult, WasmVoidResultExtension,
};
use crate::script_implementations::wasm::export_registry::{register, WasmResult};
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::MoveData;
use wasmer::FunctionEnvMut;

register! {
    fn learned_move_get_learn_method(
        env: FunctionEnvMut<WebAssemblyEnv>,
        turn_choice: ExternRef<LearnedMove>,
    ) -> WasmResult<u8> {
        unsafe {
            let learned_move = get_value_arc!(turn_choice, env);
            wasm_ok(transmute::<MoveLearnMethod, u8>(learned_move.learn_method()))
        }
    }

    fn learned_move_get_move_data(
        env: FunctionEnvMut<WebAssemblyEnv>,
        turn_choice: ExternRef<LearnedMove>,
    ) -> WasmResult<ExternRef<dyn MoveData>> {
        let turn_choice = get_value_arc!(turn_choice, env);
        wasm_ok(ExternRef::func_new(&env, turn_choice.move_data().into()))
    }

    fn learned_move_restore_all_uses(
        env: FunctionEnvMut<WebAssemblyEnv>,
        turn_choice: ExternRef<LearnedMove>,
    ) -> WasmVoidResult {
        let turn_choice = get_value_arc_void!(turn_choice, env);
        turn_choice.restore_all_uses();
        WasmVoidResult::ok()
    }

    fn learned_move_restore_uses(
        env: FunctionEnvMut<WebAssemblyEnv>,
        turn_choice: ExternRef<LearnedMove>,
        amount: u8,
    ) -> WasmVoidResult {
        let turn_choice = get_value_arc_void!(turn_choice, env);
        turn_choice.restore_uses(amount);
        WasmVoidResult::ok()
    }

}
