use crate::script_implementations::wasm::export_registry::wasm_result::{get_value, get_value_arc, wasm_ok};
use crate::script_implementations::wasm::export_registry::{register, WasmResult};
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::Item;
use crate::static_data::ItemLibrary;
use crate::StringKey;
use std::mem::transmute;
use wasmer::FunctionEnvMut;

register! {
    fn item_library_get_item(
        env: FunctionEnvMut<WebAssemblyEnv>,
        lib: ExternRef<dyn ItemLibrary>,
        string_key: ExternRef<StringKey>,
    ) -> WasmResult<ExternRef<dyn Item>> {
        let lib = get_value_arc!(lib, env);
        let string_key = get_value!(string_key, env);
        let m = lib.get(&string_key);
        wasm_ok(if let Some(v) = m {
            ExternRef::func_new(&env, (&v).into())
        } else {
            ExternRef::null()
        })
    }

    fn item_library_get_item_by_hash(env: FunctionEnvMut<WebAssemblyEnv>, lib: ExternRef<dyn ItemLibrary>, hash: u32) -> WasmResult<ExternRef<dyn Item>> {
        let lib = get_value_arc!(lib, env);
        let m = lib.get_by_hash(hash);
        wasm_ok(if let Some(v) = m {
            ExternRef::func_new(&env, (&v).into())
        } else {
            ExternRef::null()
        })
    }

    fn item_get_price(
        env: FunctionEnvMut<WebAssemblyEnv>,
        item: ExternRef<dyn Item>,
    ) -> WasmResult<i32> {
        let item = get_value_arc!(item, env);
        wasm_ok(item.price())
    }

    fn item_get_name(
        env: FunctionEnvMut<WebAssemblyEnv>,
        item: ExternRef<dyn Item>,
    ) -> WasmResult<ExternRef<StringKey>> {
        let item = get_value_arc!(item, env);
        wasm_ok(ExternRef::func_new(&env, item.name().clone().into()))
    }

    fn item_get_category(
        env: FunctionEnvMut<WebAssemblyEnv>,
        item: ExternRef<dyn Item>,
    ) -> WasmResult<u8> {
        let item = get_value_arc!(item, env);
        unsafe { wasm_ok(transmute(item.category())) }
    }

    fn item_get_battle_category(
        env: FunctionEnvMut<WebAssemblyEnv>,
        item: ExternRef<dyn Item>,
    ) -> WasmResult<u8> {
        let item = get_value_arc!(item, env);
        unsafe { wasm_ok(transmute(item.battle_category())) }
    }

    fn item_has_flag(
        env: FunctionEnvMut<WebAssemblyEnv>,
        item: ExternRef<dyn Item>,
        key: ExternRef<StringKey>
    ) -> WasmResult<u8> {
        let item = get_value_arc!(item, env);
        let key = get_value!(key, env);
        wasm_ok(if item.has_flag(&key) { 1 } else { 0 })
    }

}
