use crate::script_implementations::wasm::export_registry::wasm_result::{get_value, get_value_arc, wasm_ok};
use crate::script_implementations::wasm::export_registry::{register, WasmResult};
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::{MoveData, MoveLibrary};
use crate::StringKey;
use wasmer::FunctionEnvMut;

register! {
fn move_library_get_move(
    env: FunctionEnvMut<WebAssemblyEnv>,
    lib: ExternRef<dyn MoveLibrary>,
    string_key: ExternRef<StringKey>,
) -> WasmResult<ExternRef<dyn MoveData>> {
    let lib = get_value_arc!(lib, env);
    let string_key = get_value!(string_key, env);
    let m = lib.get(&string_key);
    wasm_ok(if let Some(v) = m {
        ExternRef::func_new(&env, (&v).into())
    } else {
        ExternRef::null()
    })
}

fn move_library_get_move_by_hash(env: FunctionEnvMut<WebAssemblyEnv>, lib: ExternRef<dyn MoveLibrary>, hash: u32) -> WasmResult<ExternRef<dyn MoveData>> {
    let lib = get_value_arc!(lib, env);
    let m = lib.get_by_hash(hash);
    wasm_ok(if let Some(v) = m {
        ExternRef::func_new(&env, (&v).into())
    } else {
        ExternRef::null()
    })
}

fn move_data_get_name(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) ->WasmResult<ExternRef<StringKey>> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(ExternRef::func_new(&env, move_data.name().clone().into()))
}

fn move_data_get_type(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> WasmResult<u8> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(move_data.move_type().into())
}
fn move_data_get_category(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> WasmResult<u8> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(move_data.category() as u8)
}
fn move_data_get_base_power(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> WasmResult<u8> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(move_data.base_power())
}
fn move_data_get_accuracy(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> WasmResult<u8> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(move_data.accuracy())
}
fn move_data_get_base_usages(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> WasmResult<u8> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(move_data.base_usages())
}
fn move_data_get_target(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> WasmResult<u8> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(move_data.target() as u8)
}
fn move_data_get_priority(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>) -> WasmResult<i8> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(move_data.priority())
}
fn move_data_has_flag(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>, flag: ExternRef<StringKey>) -> WasmResult<u8> {
    let move_data = get_value_arc!(move_data, env);
    let flag = get_value!(flag, env);
    wasm_ok(u8::from(move_data.has_flag(&flag)))
}
fn move_data_has_flag_by_hash(env: FunctionEnvMut<WebAssemblyEnv>, move_data: ExternRef<dyn MoveData>, flag_hash: u32) -> WasmResult<u8> {
    let move_data = get_value_arc!(move_data, env);
    wasm_ok(u8::from(move_data.has_flag_by_hash(flag_hash)))
}
}
