use crate::script_implementations::wasm::export_registry::wasm_result::{get_value_arc, wasm_ok};
use crate::script_implementations::wasm::export_registry::{register, WasmResult};
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::Nature;
use std::mem::transmute;
use wasmer::FunctionEnvMut;

register! {
fn nature_get_increase_stat(
    env: FunctionEnvMut<WebAssemblyEnv>,
    nature: ExternRef<dyn Nature>
) -> WasmResult<u8> {
    let nature = get_value_arc!(nature, env);
    wasm_ok(unsafe { transmute(nature.increased_stat()) })
}
fn nature_get_decrease_stat(
    env: FunctionEnvMut<WebAssemblyEnv>,
    nature: ExternRef<dyn Nature>
) -> WasmResult<u8> {
    let nature = get_value_arc!(nature, env);
    wasm_ok(unsafe { transmute(nature.decreased_stat()) })
}
fn nature_get_increase_modifier(
    env: FunctionEnvMut<WebAssemblyEnv>,
    nature: ExternRef<dyn Nature>
) -> WasmResult<f32> {
    let nature = get_value_arc!(nature, env);
    wasm_ok(nature.increased_modifier())
}
fn nature_get_decrease_modifier(
    env: FunctionEnvMut<WebAssemblyEnv>,
    nature: ExternRef<dyn Nature>
) -> WasmResult<f32> {
    let nature = get_value_arc!(nature, env);
    wasm_ok(nature.decreased_modifier())
}
}
