use crate::script_implementations::wasm::export_registry::wasm_result::{get_value, get_value_arc, wasm_ok};
use crate::script_implementations::wasm::export_registry::{register, WasmResult};
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::SpeciesLibrary;
use crate::static_data::{Form, Species};
use crate::StringKey;
use wasmer::FunctionEnvMut;

register! {

fn species_library_get_species(
    env: FunctionEnvMut<WebAssemblyEnv>,
    lib: ExternRef<dyn SpeciesLibrary>,
    string_key: ExternRef<StringKey>,
) -> WasmResult<ExternRef<dyn Species>> {
    let lib = get_value_arc!(lib, env);
    let string_key = get_value!(string_key, env);
    let m = lib.get(&string_key);
    wasm_ok(if let Some(v) = m {
        ExternRef::func_new(&env, (&v).into())
    } else {
        ExternRef::null()
    })
}

fn species_get_capture_rate(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> WasmResult<u8> {
    let species = get_value_arc!(species, env);
    wasm_ok(species.capture_rate())
}

fn species_get_growth_rate(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> WasmResult<ExternRef<StringKey>> {
    let species = get_value_arc!(species, env);
    wasm_ok(ExternRef::func_new(&env, species.growth_rate().clone().into()))
}

fn species_get_gender_rate(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> WasmResult<f32> {
    let species = get_value_arc!(species, env);
    wasm_ok(species.gender_rate())
}

fn species_get_name(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> WasmResult<ExternRef<StringKey>> {
    let species = get_value_arc!(species, env);
    wasm_ok(ExternRef::func_new(&env, species.name().clone().into()))
}

fn species_get_id(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
) -> WasmResult<u16> {
    let species = get_value_arc!(species, env);
    wasm_ok(species.id())
}

fn species_get_form_by_hash(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
    form_hash: u32
) -> WasmResult<ExternRef<dyn Form>> {
    let species = get_value_arc!(species, env);
    let form = species.get_form_by_hash(form_hash);
    wasm_ok(if let Some(form) = form {
        ExternRef::func_new(&env, (&form).into())
    } else {
        ExternRef::null()
    })
}

fn species_has_flag_by_hash(
    env: FunctionEnvMut<WebAssemblyEnv>,
    species: ExternRef<dyn Species>,
    flag_hash: u32
) -> WasmResult<u8> {
    let species = get_value_arc!(species, env);
    wasm_ok(if species.has_flag_by_hash(flag_hash) { 1 } else { 0 })
}


}
