use crate::script_implementations::wasm::export_registry::register;
use crate::script_implementations::wasm::export_registry::wasm_result::{
    get_value, get_value_arc, try_wasm, wasm_ok, WasmResult,
};
use crate::script_implementations::wasm::export_registry::FunctionEnvMut;
use crate::script_implementations::wasm::extern_ref::ExternRef;
use crate::script_implementations::wasm::script_resolver::WebAssemblyEnv;
use crate::static_data::TypeLibrary;
use crate::StringKey;

register! {

    fn type_library_get_type_by_name(
        env: FunctionEnvMut<WebAssemblyEnv>,
        lib: ExternRef<dyn TypeLibrary>,
        name: ExternRef<StringKey>
    ) -> WasmResult<u8> {
        let lib = get_value_arc!(lib, env);
        let name = get_value!(name, env);
        let type_id = lib.get_type_id(&name);
        wasm_ok(if let Some(type_id) = type_id {
            type_id.into()
        } else {
            0
        })
    }

    fn type_library_get_single_effectiveness(
        env: FunctionEnvMut<WebAssemblyEnv>,
        lib: ExternRef<dyn TypeLibrary>,
        attacking: u8,
        defending: u8
    ) -> WasmResult<f32> {
        let lib = get_value_arc!(lib, env);
        let effectiveness = try_wasm!(lib.get_single_effectiveness(attacking.into(), defending.into()), env);
        wasm_ok(effectiveness)
    }

}
