#[cfg(feature = "serde")] use serde::{Deserialize, Serialize};

/// Stats are numerical values on Pokemon that are used in battle.
#[derive(Debug, PartialEq, Eq, Copy, Clone)]
#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
#[cfg_attr(feature = "rune", derive(rune::Any))]
#[repr(u8)]
pub enum Statistic {
    /// Health Points determine how much damage a Pokemon can receive before fainting.
    #[cfg_attr(feature = "rune", rune(constructor))]
    HP,
    /// Attack determines how much damage a Pokemon deals when using a physical attack.
    #[cfg_attr(feature = "rune", rune(constructor))]
    Attack,
    /// Defense determines how much damage a Pokemon receives when it is hit by a physical attack.
    #[cfg_attr(feature = "rune", rune(constructor))]
    Defense,
    /// Special Attack determines how much damage a Pokemon deals when using a special attack.
    #[cfg_attr(feature = "rune", rune(constructor))]
    SpecialAttack,
    /// Special Defense determines how much damage a Pokemon receives when it is hit by a special attack.
    #[cfg_attr(feature = "rune", rune(constructor))]
    SpecialDefense,
    /// Speed determines the order that a Pokemon can act in battle.
    #[cfg_attr(feature = "rune", rune(constructor))]
    Speed,
}
